local M = {}

local blackhole = {["expandAccDisc"] = true, ["scale"] = 0, ["rotation"] = 0}
local reset = false
local start = false
local windX = 0
local windY = 0
local windZ = 0
local timer3 = 0

local function onScenarioChange(scenario)
	if scenario.state == 'pre-start' then
		reset = true
		start = false
		timer3 = 0
		blackhole = {["expandAccDisc"] = true, ["scale"] = 0, ["rotation"] = 0}
	end
end

local function onRaceStart()
	if not start then start = true end
end

local function updateBlackhole()
	if blackhole["expandAccDisc"] then
		blackhole["scale"] = blackhole["scale"]+0.1/2
		TorqueScript.eval([[
			accretionDisc.scale = "]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[";
		]])
		if blackhole["scale"] >= 5 then
			blackhole["expandAccDisc"] = false
		end
	end
	
	blackhole["rotation"] = blackhole["rotation"]+3
	TorqueScript.eval([[
		accretionDisc.rotation = "0 -1 0 ]]..blackhole["rotation"]..[[";
	]])
end

local function onPreRender(dt)
	if start then
		updateBlackhole()
	end
	
	if reset then
		TorqueScript.eval([[
			accretionDisc.scale = "0 0 0";
		]])
		reset = false
	end
end

local function applyGravity(vehicleObject, vehicleName)
	local blackholeObject = scenetree.findObject('scenario_player1')
	local player = map.objects[map.objectNames[vehicleName]]
	local blackhole = map.objects[map.objectNames['scenario_player1']]
	local distance = math.floor(math.abs((player.pos-blackhole.pos):length()))
	
	if vehicleObject:getPosition().x > blackholeObject:getPosition().x then 
		windX = -(750-distance)
	elseif vehicleObject:getPosition().x < blackholeObject:getPosition().x then 
		windX = (750-distance)
	end
	
	if vehicleObject:getPosition().y > blackholeObject:getPosition().y then 
		windY = -(750-distance)
	elseif vehicleObject:getPosition().y < blackholeObject:getPosition().y then 
		windY = (750-distance)
	end
	
	if vehicleObject:getPosition().z > blackholeObject:getPosition().z then 
		windZ = -(750-distance)
	elseif vehicleObject:getPosition().z < blackholeObject:getPosition().z then 
		windZ = (750-distance)
	end
	
	if     windX >  1000 then windX =  1000 
	elseif windX < -1000 then windX = -1000 end 
	if     windY >  1000 then windY =  1000 
	elseif windY < -1000 then windY = -1000 end
	if     windZ >  1000 then windZ =  1000 
	elseif windZ < -1000 then windZ = -1000 end

	local windLuaCommand = 'obj:setWind('..windX..','..windY..','..windZ..')'
	vehicleObject:queueLuaCommand(windLuaCommand)
end

local function getNumberOfVehicles(veh, vehName)
	local first = true
	local i = 0
	while true do
		if first and scenetree.findObject('clone') ~= nil then --First clone is always clone, then clone0 etc
			table.insert(veh, scenetree.findObject('clone'))
			table.insert(vehName, 'clone')
			first = false
		elseif not first and scenetree.findObject('clone'..i) then 
			table.insert(veh, scenetree.findObject('clone'..i))
			table.insert(vehName, 'clone'..i)
			i = i+1 --Clone vehicle exists, so there may be more
		else break --No clone vehicle exists therefore stop looking
		end
	end
	
	return veh, vehName
end

local function onRaceTick(raceTickTime)
	timer3 = timer3+raceTickTime
	if timer3 % 1 == 0 and blackhole["scale"] > 1 then
		local vehicles, vehicleNames = {scenetree.findObject('scenario_player0')}, {'scenario_player0'}
		vehicles, vehicleNames = getNumberOfVehicles(vehicles, vehicleNames)
		for key, value in ipairs(vehicles) do applyGravity(value, vehicleNames[key]) end
	end
end

M.onPreRender = onPreRender
M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
return M